#!/usr/bin/python

# INFO
# Dieses Skript kompiliert auf Basis eines übergeben GPX-Tracks eine spezielle Version
# der OSM-Wanderreitkarte. Dabei werden nur die Kartenkacheln verwendet, die für den
# Track benötigt werden. Das Ergebnis ist dann im Normalfall klein genug, um in den
# Speicher eines Garmin Edge 520 zu passen.

# VORRAUSSETZUNGEN
# Dieses Skript verwendet das Programm "gmt" aus dem Paket GMapTool. Dieses muss 
# heruntergeladen und installiert werden. Der Pfad zu "gmt" muss im Skript angepasst werden.
# Quelle: http://www.gmaptool.eu/en/content/gmaptool
#
# Als Quelle für die neue Karte wird "Nop's Reit- und Wanderkarte" oder kurz 
# "Wanderkarte (Nop)" verwendet. Die räumlich zum Track passende Karte muss
# heruntergeladen, entzippt und in einzelnen Kacheln zerschnitten werden. Letzters 
# kann mit "gmt" erfolgen.
# Kartendownload: http://www.wanderreitkarte.de/garmin_de.php 
# Karte zerschneiden: gmt -S wanderreitkarte.img
# 
# Für die Verarbeitung des GPX-Tracks wir die Python Bibliothek 'untangle' verwendet. 
# Diese muss ebenfalls installiert werden:  
# https://pythonadventures.wordpress.com/2011/10/30/read-xml-painlessly/
# sudo easy_install pip
# sudo pip install untangle

# AUFRUF
# mapcut.py GPX-Trackfile TileFolder OutputMapName MapName
# mapcut.py ~/tmp/gpx/track.gpx ~/tmp/wrk/ ~/tmp/map/ta17.img WRK_TA17


import sys
import re
import os
from sets import Set
import untangle

# übergebene Parameter auswerten
gpxTrackFile = sys.argv[1]
mapTileFolder = sys.argv[2]		
outputFile = sys.argv[3]
outputMapName = sys.argv[4]


# Pfad zum gmt-Binary
# Windows-Beispiele:
# gmt = "C:\Program Files (x86)\GMapTool\gmt\gmt.exe"
# gmt = "gmt.exe"
# Linux-/Mac-Beispiele
# gmt = "~/bin/gmt"
gmt = "./gmt"


# Funktion ermittelt die benötigten Kartenkacheln. Als Parameter werden
# die äußeren Grenzen des GPX-Track erwartet
def getMapTilesForSquare(minLat, maxLat, minLon, maxLon):
	minLat = float(minLat)
	maxLat = float(maxLat)
	minLon = float(minLon)
	maxLon = float(maxLon)

	# sicherstellen, dass min < max if minLon > maxLon: minLon, maxLon = maxLon, minLon 
	if minLat > maxLat: minLat, maxLat = maxLat, minLat				

	# set stellt sicher, dass jedes Element unique ist
	tiles = Set()
	f = open(mapTileFolder + "/mapset.mp", "r")	
	for line in f:
		if line.find("Label") > -1:
			# Kartenkachel-Bezeichnug/-Nummer gefunden
			matchObj = re.match( r'Label.*\]([0-9].*)',  line, re.M|re.I)
			if matchObj:
				thisTile = matchObj.group(1)

		if line.find("Data0") > -1:
			# Kartenkachel-Grenzen gefunden
			regex = r'Data0=\(([0-9]*\.[0-9]*),([0-9]*\.[0-9]*)\),\(([0-9]*\.[0-9]*),([0-9]*\.[0-9]*)\),\(([0-9]*\.[0-9]*),([0-9]*\.[0-9]*)\),\(([0-9]*\.[0-9]*),([0-9]*\.[0-9]*)\)'
			matchObj = re.match(regex, line, re.M|re.I)
			if matchObj:
				tileMinLat = float(matchObj.group(1))
				tileMaxLat = float(matchObj.group(5))
				tileMinLon = float(matchObj.group(2))
				tileMaxLon = float(matchObj.group(4))

				# sicherstellen, dass min < max if tileMinLon > tileMaxLon: tileMinLon, tileMaxLon = tileMaxLon, tileMinLon 
				if tileMinLat > tileMaxLat: tileMinLat, tileMaxLat = tileMaxLat, tileMinLat 

				# Jetzt prüfen, ob wir diese Kachel für unseren Track benötigen

				# Möglichkeit 1: eine der Track-Grenzen liegt innerhalb der Kachel
				if (tileMinLon <= minLon <= tileMaxLon) and (tileMinLat <= maxLat <= tileMaxLat):
					tiles.add(thisTile)
				if (tileMinLon <= minLon <= tileMaxLon) and (tileMinLat <= minLat <= tileMaxLat):
					tiles.add(thisTile)
				if (tileMinLon <= maxLon <= tileMaxLon) and (tileMinLat <= maxLat <= tileMaxLat):
					tiles.add(thisTile)					
				if (tileMinLon <= maxLon <= tileMaxLon) and (tileMinLat <= minLat <= tileMaxLat):
					tiles.add(thisTile)					

				# Möglichkeit 2: die Kachel liegt innerhalb der Track-Grenzen
				if (minLon <= tileMinLon <= maxLon) and (minLat <= tileMaxLat <= maxLat):
					tiles.add(thisTile)
				if (minLon <= tileMinLon <= maxLon) and (minLat <= tileMinLat <= maxLat):
					tiles.add(thisTile)
				if (minLon <= tileMaxLon <= maxLon) and (minLat <= tileMaxLat <= maxLat):
					tiles.add(thisTile)					
				if (minLon <= tileMaxLon <= maxLon) and (minLat <= tileMinLat <= maxLat): tiles.add(thisTile) return tiles # GPX-Track analysieren und Grenzen ermitteln # Länge/ Longitude minLon=180.0 # Laengengrad > 0 (Greenitch)
maxLon=0.0	# Laengengrad > 0 (Greenitch)
# Breite/ Latitude - Nordhalbkugel
minLat=90.0	# Breitengrad > 0 (Aequator)
maxLat=0.0	# Breitengrad  maxLat:
					maxLat = lat
				if lon < minLon: minLon = lon if lon > maxLon:
					maxLon = lon
	
print ("Grenzen des Track erfolgreich ermittelt: ")
print ("  Länge/Longitude: " + str(minLon) + " -> " + str(maxLon))
print ("  Breite/Latitude: " + str(minLat) + " -> " + str(maxLat))


# Benötigte Kacheln ermitteln...
tiles = Set()
tiles = getMapTilesForSquare(minLat,maxLat,minLon,maxLon)
# ...und ausgeben
print tiles


# Kommando zum Kompilieren der Karte zusammensetzen
tmp = ""
for tile in tiles:
	tmp = tmp + mapTileFolder + "/" + tile + ".img "

# TODO: -f 82,1 könnte dynamisch sein und muss (vorher?) z.B. mit  'gmt -i wanderreitkarte.img'  ermittelt werden
cmd  = gmt + " -j -f 82,1 -o " + outputFile + " -m " + outputMapName + " " + mapTileFolder + "/*.TYP" + " " + tmp

# Generierten Befehl ausgeben...
print cmd

# ...und die Karte kompilieren // dazu ggf. die nachfolgenden Zeile einkommentieren ;) 
# os.system(cmd)


